// Written by Robert C. Kirby
// Copyright 2005 by The University of Chicago
// Distributed under the LGPL license
// This work is partially supported by the US Department of Energy
// under award number DE-FG02-04ER25650

#ifndef EXPANSIONS_H
#define EXPANSIONS_H
#include <cmath>
#include <blitz/array.h>
#include <vector>
#include "FIAT_exception.hpp"
#include "FIAT_shapes.hpp"
#include "FIAT_shapes.hpp"
namespace FIAT {

  using namespace std;

  class Expansions;

  typedef blitz::Array<double,1> (*coord_transform)( const blitz::Array<double,1> &);
  typedef blitz::Array<double,2> (*tabulator)( int , const blitz::Array<double,2> & );
  typedef blitz::Array<double,3> (*deriv_tabulator)( int , const blitz::Array<double,2> & );

  class Expansions
  {

  public:
    // Primary expansion functions introduced by Karniadakis & Sherwin
    static double psitilde_a(int,double);
    static double psitilde_b(int,int,double);
    static double psitilde_c(int,int,int,double);
    // for a vector of values eta, tabulates (0.5*(1-eta))**i for
    // each 0\leq i \leq n
    static blitz::Array<double,2> make_scalings(int,const blitz::Array<double,1>);
			  
    // conversion between collapsed (eta) and rectangular (xi) coordinates
    static coord_transform coord_change( int shape );
    static coord_transform inverse_coord_change( int shape );
    static tabulator get_tabulator( int );
    static deriv_tabulator get_deriv_tabulator( int );

    static blitz::Array<double,1> eta_triangle( const blitz::Array<double,1> &xi ) ;
    static blitz::Array<double,1> xi_triangle( const blitz::Array<double,1> &eta ) ;

    static blitz::Array<double,1> eta_tetrahedron( const blitz::Array<double,1> &xi ) ;
    static blitz::Array<double,1> xi_tetrahedron( const blitz::Array<double,1> &eta ) ;

  };
}

#endif
