// Written by Robert C. Kirby
// Copyright 2005 by The University of Chicago
// Distributed under the LGPL license
// This work is partially supported by the US Department of Energy
// under award number DE-FG02-04ER25650

#ifndef QUADRATURE_H
#define QUADRATURE_H
#include <cmath>
#include <exception>
#include "FIAT_jacobi.hpp"
#include "FIAT_shapes.hpp"
#include "FIAT_expansions.hpp"
#include "FIAT_rcp.hpp"
#include <blitz/array.h>

namespace FIAT {

  class QuadratureRule;
  class JacobiQuadratureRule;
  class GaussQuadratureRule;
  class CollapsedQuadratureRuleTriangle;
  class CollapsedQuadratureRuleTetrahedron;

  RCP<QuadratureRule> make_collapsed_quadrature( int, int);
  
  class QuadratureRule
  {
  public:
    QuadratureRule() {;}
    QuadratureRule( blitz::Array<double,2> x , blitz::Array<double,1> w ) :
      x(x) , w(w) {}
    blitz::Array<double,2> get_points() const { return x; }
    blitz::Array<double,1> get_weights() const { return w; }

  protected:
    blitz::Array<double,2> x;
    blitz::Array<double,1> w;
  };

  class JacobiQuadratureRule : public QuadratureRule
  {
  public:
    JacobiQuadratureRule( double a , double b , int m );
  };

  class GaussQuadratureRule : public JacobiQuadratureRule
  {
  public: 
    GaussQuadratureRule( int m ) : JacobiQuadratureRule(0,0,m) {}
  };

  class CollapsedQuadratureRuleTriangle : public QuadratureRule
  {
  public:
    CollapsedQuadratureRuleTriangle( int m );
  };


  class CollapsedQuadratureRuleTetrahedron : public QuadratureRule
  {
  public:
    CollapsedQuadratureRuleTetrahedron( int m );
  };
}
#endif

