// Written by Robert C. Kirby
// Copyright 2005 by The University of Chicago
// Distributed under the LGPL license
// This work is partially supported by the US Department of Energy
// under award number DE-FG02-04ER25650


#ifndef RCP_H
#define RCP_H
#include "FIAT_exception.hpp"

namespace FIAT {

  using namespace std;

  template<class T>
  class RCP;

  template<class T>
  class RCP
  {
  public:
    RCP(T *t = 0) : t_(t),rc_(0)
    {
      if (t == 0) {
        t_ = 0; rc_ = 0;
      }
      else {
        t_ = t;
        rc_ = new int;
        if (!rc_) {
          FIAT_ERROR("Error allocating in rcp constructor");
        }
        *rc_ = 1;
      }
    }
    
    RCP( const RCP<T> &other ) : t_( 0 ) , rc_( 0 )
    {
      if (!other.isNull()) {
        t_ = other.t_;
        rc_ = other.rc_;
        (*rc_)++;
      }
    }
    
    ~RCP()
    {
      if (t_ != 0) {
        if (rc_ == 0) {
          FIAT_ERROR("nonnull data with null reference count");
        }
        else { // t_ and rc_ nonnull
          --(*rc_);
          if (*rc_ == 0) {
            delete t_; delete rc_;
            t_ = 0; rc_ = 0;
          }
        }
      }
      else if (rc_ != 0) {
        FIAT_ERROR("null data with nonnull reference count");
      }
    }
    
    const RCP<T> &operator=(const RCP<T> &other)
    {
      if (t_ != other.t_) {
        if (rc_ != 0) {
          --(*rc_);
          if (*rc_ == 0) {
            delete t_; delete rc_;
            t_ = 0; rc_ = 0;
          }
        }
        t_ = other.t_;
        rc_ = other.rc_;
        (*rc_)++;
      }
      
      return *this;
    }
    
    bool isNull() const {return t_==0;}
    T* operator->() const
    {
      if (!t_) throw exception();
      else return t_;
    }

    T& operator*() const
    {
      if (!t_) throw exception();
      else return *t_;
    }


  private:
    T *t_;
    int *rc_;
  };
}
#endif
